library(matlab)

path.id<-"E:\\Wind Power\\Spline"
sub.id<-"Output"
dir.create(file.path(path.id,sub.id),showWarnings=F)

source(file.path(paste(path.id,"\\Code\\binning.r",sep="")))

### Read data
file.id<-"ILT3"

in.path1<-file.path(paste(path.id,"\\Data\\DATA_",file.id,".txt",sep=""))
in.path2<-file.path(paste(path.id,"\\Output\\sample_vs_",file.id,".txt",sep=""))
data<-read.table(in.path1,header=T)
ws<-read.table(in.path2,header=T)

bin.size<-c(2,0.5)
n.rep<-1000		# To construct credible interval
n.sp<-100		# To sample y from gumbel given mu and sigma
n.yr<-c(20,50)
prob.t<-10/(n.yr*365.25*24*60)

param.sp<-sample.MLE(data,bin.size,n.rep)	# Sample params 4 each bin from MLE
bin.ts<-bin.div(ws,bin.size)[[1]]	# Get the bin index for each sample pt
n.obs<-dim(bin.ts)[1]
lt<-matrix(0,n.rep,2)

tic()
for(i in 1:n.rep){
	mu<-param.sp[bin.ts[,1],i]
	sigma<-param.sp[bin.ts[,1],n.rep+i]
	y.est<-rgumbel(n.sp*n.obs,mu,sigma)
	y.srt<-sort(y.est,decreasing=T)
	lt[i,]<-y.srt[prob.t*n.sp*n.obs]
}
toc()

### Results
lt.srt1<-sort(lt[,1])	# lt with 20 yrs
lt.srt2<-sort(lt[,2])	# lt with 50 yrs

CI20.med<-c(median(lt[,1]),lt.srt1[25],lt.srt1[975])
CI50.med<-c(median(lt[,2]),lt.srt2[25],lt.srt2[975])

### Table 5 and 6
write.table(rbind(CI20.med,CI50.med),paste(path.id,"\\Output\\CI_bin_",
	file.id,".txt",sep=""))
