### Bayesian MARS: select model params randomly
### Get a basis function represented by a column in the basis matrix
get.basisFn<-function(X,Xt,interaction,order){
	n<-dim(X)[1]
	p<-dim(X)[2]			## number of input vars. for Basis fn
	nt<-dim(Xt)[1]
	depth<-interaction+1
	knot.pos<-matrix(0,depth,1)
	sign.ind<-matrix(-1,depth,1)	# to contain T,F values later (T:1, F:0)
	seg.fn<-matrix(0,n,depth)
	seg.fn_t<-matrix(0,nt,depth)
	B.k<-matrix(NA,n,1)
	B.k_t<-matrix(NA,nt,1)
	not_finished<-1
	while(not_finished==1){
		var<-matrix(0,depth,1)
		for(j in 1:depth){		# for each variable
			obs.id<-ceiling(runif(1,0,1)*n)
			var.selected<-0
			while(var.selected==0){
				param.id<-ceiling(runif(1,0,1)*p)
				if(!is.element(param.id,var[1:j])){	# only for
					var[j]<-param.id			# the var. not
					var.selected<-1			# used yet
				}
			}
			sign.ind[j]<-runif(1,0,1)>0.5	# assign the sign value
			knot.pos[j]<-X[obs.id,var[j]]	# knot pos. for the cur. var.
		
			{if(sign.ind[j]==0){	# when h -, only min takes effect
				seg.fn[,j]<-apply(cbind(0,X[,var[j]]-knot.pos[j]),
					1,min)
				seg.fn_t[,j]<-apply(cbind(0,Xt[,var[j]]-knot.pos[j]),
					1,min)
			}
			else{				# when h +, only max takes effect
				seg.fn[,j]<-apply(cbind(0,X[,var[j]]-knot.pos[j]),
					1,max)
				seg.fn_t[,j]<-apply(cbind(0,Xt[,var[j]]-knot.pos[j]),
					1,max)
			}}
		}

		{if(order==0){
			seg.fn<-seg.fn!=0		# if order is 0, assign 1's (const.)
			seg.fn_t<-seg.fn_t!=0
		}
		else{
			seg.fn<-seg.fn^order
			seg.fn_t<-seg.fn_t^order
		}}

		B.k<-apply(seg.fn,1,prod)
		B.k_t<-apply(seg.fn_t,1,prod)
		not_finished<-prod(B.k==0)
	}
	mean_B.k<-mean(B.k)
	std_B.k<-sd(B.k)
	B.k<-(B.k-mean_B.k)/std_B.k
	B.k_t<-(B.k_t-mean_B.k)/std_B.k
	list(B.k,B.k_t)
}