library(VGAM)
library(mnormt)

### Division of bins
bin.div<-function(data,bin.size){
	v<-data[,1]
	s<-data[,2]
	n<-length(v)
	bin.size.v<-bin.size[1]
	bin.size.s<-bin.size[2]
	n.bin.v<-ceiling((24-5)/bin.size.v)
	n.bin.s<-ceiling((3-0.25)/bin.size.s)
	ref.v<-rep(seq(5,24,by=bin.size.v),n.bin.s)
	ref.s<-rep(seq(0.25,3,by=bin.size.s),each=n.bin.v)
	ref.ind<-c(1:(n.bin.v*n.bin.s))
	ref<-cbind(ref.v,ref.s,ref.ind)
	index.v<-trunc((v-4)/bin.size.v)+1
	index.s<-trunc((s)/bin.size.s)+1
	index.v[which(index.v>n.bin.v)]<-n.bin.v
	index.s[which(index.s>n.bin.s)]<-n.bin.s
	index<-index.v+(index.s-1)*n.bin.v
	data.new<-cbind(index,data)
	list(data.new,ref)
}

### MLE estimation using Gumbel distribution
bin.MLE<-function(data,bin.size){
	bin<-bin.div(data,bin.size)
	data.new<-bin[[1]]
	ref<-bin[[2]]
	param.est<-matrix(0,dim(ref)[1],10)
	for(i in 1:dim(ref)[1]){
		t<-which(data.new[,1]==i)
		data.bin<-data.new[t,]
		if(dim(data.bin)[1]>5){
			lik.Gum<-function(param){
				mu<-param[1]
				sigma<-exp(param[2])
				-sum(log(dgumbel(data.bin[,4],mu,sigma)))
			}
			param.init<-rep(0.1,2)
			mle.Gum<-nlm(lik.Gum,param.init,hessian=T,stepmax=10,iterlim=1000)
			param.est[i,4]<-dim(data.bin)[1]
			param.est[i,5:6]<-rmnorm(1,mle.Gum$est,solve(mle.Gum$hessian))
		}
	}
	param.est[,6]<-exp(param.est[,6])
	param.est[,1:3]<-ref[,1:3]
	
	### Estimation for empty bins
	id.exist<-which(param.est[,4]>0)
	est1<-param.est[id.exist,]
	est2<-param.est[-id.exist,]

	for(j in 1:dim(est2)[1]){
		dist<-1/(((est2[j,1]-est1[,1])/bin.size[1])^2
			+((est2[j,2]-est1[,2])/bin.size[2])^2)^0.5
		weight<-dist/sum(dist)
		est2[j,5]<-sum(weight*est1[,5])
		est2[j,6]<-sum(weight*est1[,6])
	}
	param.est[-id.exist,]<-est2

	return(param.est)
}

### Draw sample parameters from the MLE distributions
sample.MLE<-function(data,bin.size,n.rep){
	bin<-bin.div(data,bin.size)
	data.new<-bin[[1]]
	ref<-bin[[2]]
	#param.est<-rep(list(matrix()),dim(ref)[1])
	param.est<-matrix(0,dim(ref)[1],2*n.rep)
	bin.info<-matrix(0,dim(ref)[1],4)
	for(i in 1:dim(ref)[1]){
		t<-which(data.new[,1]==i)
		data.bin<-data.new[t,]
		if(dim(data.bin)[1]>5){
			lik.Gum<-function(param){
				mu<-param[1]
				sigma<-exp(param[2])
				-sum(log(dgumbel(data.bin[,4],mu,sigma)))
			}
			param.init<-rep(0.1,2)
			mle.Gum<-nlm(lik.Gum,param.init,hessian=T,stepmax=10,
				iterlim=1000)
			bin.info[i,4]<-dim(data.bin)[1]
			samples<-rmnorm(n.rep,mle.Gum$est,
				solve(mle.Gum$hessian))
			param.est[i,]<-c(samples[,1],exp(samples[,2]))
		}
	}
	bin.info[,1:3]<-ref
	params<-neighbor.est(list(bin.info,param.est))
	return(params)
}

### Estimate parameters for the empty (or <=5) bins
neighbor.est<-function(samples){
	ref<-samples[[1]]
	params<-samples[[2]]
	index.exist<-which(ref[,4]>5)
	index.empty<-which(ref[,4]<=5)
	bin.exist<-ref[which(ref[,4]>5),]
	n.exist<-dim(bin.exist)[1]

	for(i in index.empty){
		loc<-matrix(rep((ref[i,1:2]),n.exist),n.exist,2,byrow=T)
		dist<-1/(((bin.exist[,1]-loc[,1])/bin.size[1])^2
			+((bin.exist[,2]-loc[,2])/bin.size[2])^2)^0.5
		weight<-dist/sum(dist)
		for(j in 1:n.rep){
			params[i,j]<-sum(weight*params[index.exist,j])
			params[i,j+n.rep]<-sum(weight*params[index.exist,j+n.rep])
		}
	}
	return(params)
}