library(mnormt)
library(matlab)

path.id<-"E:\\Wind Power\\Spline"
sub.id<-"Output"
dir.create(file.path(path.id,sub.id),showWarnings=F)

source(file.path(paste(path.id,"\\Code\\mle_y.r",sep="")))
source(file.path(paste(path.id,"\\Code\\basisFn.r",sep="")))
source(file.path(paste(path.id,"\\Code\\binning.r",sep="")))
source(file.path(paste(path.id,"\\Code\\GPL.r",sep="")))

### Read data
file.id<-"ILT2"
data.path<-file.path(paste(path.id,"\\Data\\DATA_",file.id,".txt",sep=""))
x.path<-file.path(paste(path.id,"\\Output\\Xmars_",file.id,".txt",sep=""))
r.path<-file.path(paste(path.id,"\\Output\\Rmars_",file.id,".txt",sep=""))

data<-read.table(data.path)
x.mars.init<-as.matrix(read.table(x.path))
r.mars.init<-as.matrix(read.table(r.path))

### Set parameters
n.obs<-dim(data)[1]
n.test<-round(0.2*n.obs)
n.train<-n.obs-n.test

prob<-c(seq(0.900,0.999,by=0.001))
n.prob<-length(prob)
n.rep<-10

### Comparable stats
bin.est<-matrix(NA,n.test,n.prob)
gev.est<-matrix(NA,n.test,n.prob)

bin.GPL<-rep(list(matrix(NA,n.rep,n.prob)),3)
gev.GPL<-rep(list(matrix(NA,n.rep,n.prob)),3)

yt<-matrix(NA,n.rep,n.test)

### Replication of comparison
tic()
for(rep in 1:n.rep){
	
	### Random data selection
	id.test<-sample(c(1:n.obs),n.test)
	id.train<-c(1:n.obs)[-id.test]
	TR<-data[id.train,]
	TS<-data[id.test,]
	x.init<-TR[,1:2]
	y<-TR[,3]
	xt.init<-TS[,1:2]
	yt[rep,]<-TS[,3]

	### Binning method ##############################################
	for(i in 1:100){
		bin.test<-bin.div(TS,c(2,0.5))[[1]]
		TR.param<-bin.MLE(TR,c(2,0.5))
		TS.param<-TR.param[bin.test[,1],]
		
		bin<-matrix(NA,n.test,n.prob)
		for(j in 1:n.prob){
			bin[,j]<-qgumbel(prob[j],TS.param[,5],TS.param[,6])
		}
		
		if(i==1){
			bin.est<-bin}
		else{
			bin.est<-(((i-1)*bin.est)+bin)/i}
	}

	### Spline method ###############################################
	### Set experiment parameters
	n.mcmc<-100
	burn.in<-10
	
	p<-dim(xt.init)[2]

	### Standardize data
	mean.x<-rep(1,p)
	sd.x<-rep(1,p)
	x<-x.init
	xt<-xt.init
	for(i in 1:p){
		mean.x[i]<-mean(x.init[,i])
		sd.x[i]<-sd(x.init[,i])
		x[,i]<-(x.init[,i]-mean.x[i])/sd.x[i]
		xt[,i]<-(xt.init[,i]-mean.x[i])/sd.x[i]
	}
	
	x<-as.matrix(x)
	xt<-as.matrix(xt)

	### Set MARS parameters
	beta.int<-0		###################################### for ILT2 only
	theta.int<-0
	beta.ord<-c(0,1)
	theta.ord<-c(0,1)
	n.int1<-length(beta.int)
	n.int2<-length(theta.int)
	n.ord1<-length(beta.ord)
	n.ord2<-length(theta.ord)

	x.mars<-x.mars.init[id.train,]
	r.mars<-r.mars.init[id.train,]
	xt.mars<-x.mars.init[id.test,]
	rt.mars<-r.mars.init[id.test,]

	k1<-dim(x.mars)[2]
	k2<-dim(r.mars)[2]
	k1.max<-50
	k2.max<-20

	### Get initial MLE of y
	res.mle<-mle.y_vs(y,x.mars,r.mars)
	marg_ll<-as.numeric(res.mle[[1]])
	mle.y.est<-c(res.mle[[2]])
	mle.y.sd<-c((diag(res.mle[[3]])^.5))
	mle.y.cov<-as.matrix(res.mle[[3]])
	mle.y.res<-as.numeric(res.mle[[4]])

	### MCMC replication
	count<-0
	sample<-0

	move.cnt1<-matrix(0,1,3)
	move.cnt2<-matrix(0,1,3)
	acc.cnt1<-matrix(0,1,3)
	acc.cnt2<-matrix(0,1,3)

	while(sample<n.mcmc){
		count<-count+1
		x.new<-as.matrix(x.mars[,1:k1])
		xt.new<-as.matrix(xt.mars[,1:k1])
		r.new<-as.matrix(r.mars[,1:k2])
		rt.new<-as.matrix(rt.mars[,1:k2])

		k1.new<-k1
		k2.new<-k2

		move1<-0		# 1:Birth 2:Death 3:Move
		move2<-0		# 1:Birth 2:Death 3:Move
		flag1<-0
		flag2<-0

		### Move selection
		u1<-runif(1,0,1)
		{if(u1<(1/3)){
			move1<-1
			flag1<-1
			if(k1==k1.max){
				move1<-3
				flag1<-3
			}
		}
		else if(u1<(2/3)){
			move1<-2
			flag1<-2
			if(k1==1){
				move1<-0
				flag1<-3
			}
		}
		else{
			move1<-3
			flag1<-3
			if(k1==1){
				move1<-0
			}
		}}

		u2<-runif(1,0,1)
		{if(u2<(1/3)){
			move2<-1
			flag2<-1
			if(k2==k2.max){
				move2<-3
				flag2<-3
			}
		}
		else if(u2<(2/3)){
			move2<-2
			flag2<-2
			if(k2==1){
				move2<-0
				flag2<-3
			}
		}
		else{
			move2<-3
			flag2<-3
			if(k2==1){
				move2<-0
			}
		}}
	
		move.cnt1[flag1]<-move.cnt1[flag1]+1
		move.cnt2[flag2]<-move.cnt2[flag2]+1

		### Take an action
		{if(move1==1){	# Birth
			k1.new<-k1+1
			id.int<-ceiling(runif(1,0,1)*n.int1)
			id.ord<-ceiling(runif(1,0,1)*n.ord1)
			basis.beta<-get.basisFn(x,xt,beta.int[id.int],beta.ord[id.ord])
			x.new<-cbind(x.new,basis.beta[[1]])
			xt.new<-cbind(xt.new,basis.beta[[2]])
		}
		else if(move1==2){	# Death
			k1.new<-k1-1
			id.del<-ceiling(runif(1,0,1)*(k1-1))+1
			x.new<-x.new[,-id.del]
			xt.new<-xt.new[,-id.del]
		}
		else if(move1==3){	# Move
			id.del<-ceiling(runif(1,0,1)*(k1-1))+1
			id.int<-ceiling(runif(1,0,1)*n.int1)
			id.ord<-ceiling(runif(1,0,1)*n.ord1)
			basis.beta<-get.basisFn(x,xt,beta.int[id.int],beta.ord[id.ord])
			x.new[,id.del]<-basis.beta[[1]]
			xt.new[,id.del]<-basis.beta[[2]]
		}}

		{if(move2==1){	# Birth
			k2.new<-k2+1
			id.int<-ceiling(runif(1,0,1)*n.int2)
			id.ord<-ceiling(runif(1,0,1)*n.ord2)
			basis.theta<-get.basisFn(as.matrix(x[,1]),as.matrix(xt[,1]),
				theta.int[id.int],theta.ord[id.ord])	### T_k = 1 
			r.new<-cbind(r.new,basis.theta[[1]])
			rt.new<-cbind(rt.new,basis.theta[[2]])
		}
		else if(move2==2){	# Death
			k2.new<-k2-1
			id.del<-ceiling(runif(1,0,1)*(k2-1))+1
			r.new<-r.new[,-id.del]
			rt.new<-rt.new[,-id.del]
		}
		else if(move2==3){	# Move
			id.del<-ceiling(runif(1,0,1)*(k2-1))+1
			id.int<-ceiling(runif(1,0,1)*n.int2)
			id.ord<-ceiling(runif(1,0,1)*n.ord2)
			basis.theta<-get.basisFn(as.matrix(x[,1]),as.matrix(xt[,1]),
				theta.int[id.int],theta.ord[id.ord])	### T_k = 1 
			r.new[,id.del]<-basis.theta[[1]]
			rt.new[,id.del]<-basis.theta[[2]]
		}}

		### Check acceptance of phi sub mu star
		res.mle.beta<-mle.y_vs(y,x.new,r.mars[,1:k2])
		marg_ll.beta<-as.numeric(res.mle.beta[[1]])
		mle.y.beta.est<-c(res.mle.beta[[2]])
		mle.y.beta.sd<-c((diag(res.mle.beta[[3]])^.5))
		mle.y.beta.cov<-as.matrix(res.mle.beta[[3]])
		mle.y.beta.res<-as.numeric(res.mle.beta[[4]])

		if(mle.y.beta.res<4){
			if(runif(1,0,1)<exp(marg_ll.beta-marg_ll)){
				marg_ll<-marg_ll.beta
				mle.y.est<-mle.y.beta.est
				mle.y.sd<-mle.y.beta.sd
				mle.y.cov<-mle.y.beta.cov
				k1<-k1.new
				x.mars<-as.matrix(x.new)
				xt.mars<-as.matrix(xt.new)
				acc.cnt1[flag1]<-acc.cnt1[flag1]+1
			}
		}

		### Check acceptance of phi sub sigma star
		res.mle.theta<-mle.y_vs(y,x.mars[,1:k1],r.new)
		marg_ll.theta<-as.numeric(res.mle.theta[[1]])
		mle.y.theta.est<-c(res.mle.theta[[2]])
		mle.y.theta.sd<-c((diag(res.mle.theta[[3]])^.5))
		mle.y.theta.cov<-as.matrix(res.mle.theta[[3]])
		mle.y.theta.res<-as.numeric(res.mle.theta[[4]])

		if(mle.y.theta.res<4){
			if(runif(1,0,1)<exp(marg_ll.theta-marg_ll)){
				marg_ll<-marg_ll.theta
				mle.y.est<-mle.y.theta.est
				mle.y.sd<-mle.y.theta.sd
				mle.y.cov<-mle.y.theta.cov
				k2<-k2.new
				r.mars<-as.matrix(r.new)
				rt.mars<-as.matrix(rt.new)
				acc.cnt2[flag2]<-acc.cnt2[flag2]+1
			}
		}
	
		### Take a posterior sample
		if(count>burn.in){
			beta<-rmnorm(1,mle.y.est[1:k1],mle.y.cov[1:k1,1:k1])
			if(k2>1){
				theta<-rmnorm(1,mle.y.est[(k1+1):(k1+k2)],
					mle.y.cov[(k1+1):(k1+k2),(k1+1):(k1+k2)])
			}
			else{
				theta<-rnorm(1,mle.y.est[(k1+1):(k1+k2)],
					mle.y.sd[(k1+1):(k1+k2)])
			}

			mu<-xt.mars[,1:k1]%*%t(beta)
			sigma<-exp(rt.mars[,1:k2]%*%t(theta))
			xi<-rnorm(1,mle.y.est[(k1+k2+1)],mle.y.sd[(k1+k2+1)])

		
			gev<-matrix(NA,n.test,n.prob)	
			for(j in 1:n.prob){
				gev[,j]<-qgev(prob[j],mu,sigma,xi)		# check!
			}

			sample<-sample+1

			if(sample==1){
				gev.est<-gev
			}
			else{
				gev.est<-(((i-1)*gev.est)+gev)/i
			}
		}
	}

	for(j in 1:n.prob){
		bin.GPL[[1]][rep,j]<-get.GPL(0,j,prob,yt[rep,],bin.est)
		gev.GPL[[1]][rep,j]<-get.GPL(0,j,prob,yt[rep,],gev.est)
		bin.GPL[[2]][rep,j]<-get.GPL(1,j,prob,yt[rep,],bin.est)
		gev.GPL[[2]][rep,j]<-get.GPL(1,j,prob,yt[rep,],gev.est)
		bin.GPL[[3]][rep,j]<-get.GPL(2,j,prob,yt[rep,],bin.est)
		gev.GPL[[3]][rep,j]<-get.GPL(2,j,prob,yt[rep,],gev.est)
	}
}
toc()

### Figure 7
pdf(paste(path.id,"\\Output\\Fig7_",file.id,".pdf",sep=""))
par(mar=c(5,5,3,5))
percent<-(apply(bin.GPL[[2]][,1:91],2,mean)-apply(gev.GPL[[2]][,1:91],2,mean))/apply(bin.GPL[[2]][,1:91],2,mean)*100
plot(prob[1:91],apply(gev.GPL[[2]][,1:91],2,mean),xlab="Probability (tau)",ylab="",cex.lab=2.3,cex.axis=1.8,lwd=4,lty=2,type="l",col=1,ylim=c(0.0,0.012))
points(prob[1:91],apply(bin.GPL[[2]][,1:91],2,mean),xlab="Probability (tau)",ylab="",cex.lab=2.3,cex.axis=1.8,lwd=4,lty=3,type="l",col=2,ylim=c(0.0,0.012))
mtext(2,text="Mean score",line=3,cex=2.3)
par(new=T) 
plot(prob[1:91],percent,xlab="Probability (tau)",ylab="",cex.lab=2.3,cex.axis=1.8,lwd=4,lty=1,type="l",col=3,ylim=c(0,30),yaxt="n")
axis(4,ylim=c(0,30),cex.axis=1.8)
mtext(4,text="Percentage",line=3,cex=2.3)
legend("topleft",c("Spline method","Binning method","Perctentage value"),lty=c(2,3,1),col=c(1,2,3),lwd=5,cex=2.0) 
dev.off()

### Table 3 results
pow0_9q<-c(mean(bin.GPL[[1]][,1]),mean(gev.GPL[[1]][,1]))
pow1_9q<-c(mean(bin.GPL[[2]][,1]),mean(gev.GPL[[2]][,1]))
pow2_9q<-c(mean(bin.GPL[[3]][,1]),mean(gev.GPL[[3]][,1]))
table3<-as.data.frame(rbind(pow0_9q,pow1_9q,pow2_9q))
names(table3)<-c("Binning","Spline")
write.table(table3,paste(path.id,"\\Output\\Table3_",file.id,".txt",sep=""))

### Table 4 results
pow0_99q<-c(mean(bin.GPL[[1]][,91]),mean(gev.GPL[[1]][,91]))
pow1_99q<-c(mean(bin.GPL[[2]][,91]),mean(gev.GPL[[2]][,91]))
pow2_99q<-c(mean(bin.GPL[[3]][,91]),mean(gev.GPL[[3]][,91]))
table4<-as.data.frame(rbind(pow0_99q,pow1_99q,pow2_99q))
names(table4)<-c("Binning","Spline")
write.table(table4,paste(path.id,"\\Output\\Table4_",file.id,".txt",sep=""))
