library(ismev)
library(mnormt)
library(VGAM)
library(matlab)
library(fields)

path.id<-"E:\\Wind Power\\Spline"
sub.id<-"Output"
dir.create(file.path(path.id,sub.id),showWarnings=F)
file.id<-"ILT2"

source(file.path(paste(path.id,"\\Code\\binning.r",sep="")))
source(file.path(paste(path.id,"\\Code\\mle_y.r",sep="")))
data<-read.table(file.path(paste(path.id,"\\Data\\DATA_",file.id,".txt",sep="")))
xt.init<-read.table(file.path(paste(path.id,"\\Output\\sample_vs_",file.id,
	".txt",sep="")))

yrs<-c(20,50)
prob<-c(seq(0.900,0.999,by=0.001),(1-10/(yrs*365.25*24*60)))
power<-seq(0,2.5,by=0.1)

x.init<-data[,1:2]
y<-data[,3]

n.prob<-length(prob)
n.rep<-1000
n.sp<-100
n.obs<-dim(x.init)[1]
nt.obs<-dim(xt.init)[1]
p<-dim(xt.init)[2]

### Binning parameters
bin.size<-c(2,0.5)
bin.tr<-bin.div(data,bin.size)
bin.ts<-bin.div(xt.init,bin.size)[[1]]
param.bin<-bin.MLE(data,bin.size)

### Spline estimation
x.mars<-as.matrix(read.table(file.path(paste(path.id,
	"\\Output\\Xmars_",file.id,".txt",sep=""))))
r.mars<-as.matrix(read.table(file.path(paste(path.id,
	"\\Output\\Rmars_",file.id,".txt",sep=""))))
xt.mars<-as.matrix(read.table(file.path(paste(path.id,
	"\\Output\\Xtmars_",file.id,".txt",sep=""))))
rt.mars<-as.matrix(read.table(file.path(paste(path.id,
	"\\Output\\Rtmars_",file.id,".txt",sep=""))))
k1<-dim(x.mars)[2]
k2<-dim(r.mars)[2]
res.mle<-mle.y_vs(y,x.mars,r.mars)
marg_ll<-as.numeric(res.mle[[1]])
mle.y.est<-c(res.mle[[2]])
beta.est<-mle.y.est[1:k1]
theta.est<-mle.y.est[(k1+1):(k1+k2)]
xi.spline<-mle.y.est[(k1+k2+1)]

comp<-rep(0,60)	# 60 is the total number of bins
for(i in 1:60){
	obs.id<-which(bin.ts[,1]==i)
	if(length(obs.id)>0){
		mu.spline<-apply(xt.mars[obs.id,1:k1]%*%beta.est,2,mean)
		sigma.spline<-apply(exp(rt.mars[obs.id,1:k2]%*%theta.est),2,mean)
		comp[i]<-(qgev(0.99,mu.spline,sigma.spline,xi.spline)-qgev(0.99,
			param.bin[i,5],param.bin[i,6]))
	}
}

### Figure 8
comp<-abs(comp)
lon<-as.numeric(levels(factor(bin.tr[[2]][,1])))
lat<-as.numeric(levels(factor(bin.tr[[2]][,2])))
image<-matrix(comp/max(comp),nrow=length(lon),ncol=length(lat),byrow=F)
out.path<-file.path(paste(path.id,"\\Output\\Fig8_",file.id,".pdf",sep=""))
pdf(out.path)
par(mar=c(6,6,5,3))
image.plot(lon,lat,image,col=gray(c(10:0)/10),
	xlab="10-min average wind speed",ylab="standard deviation during 10-min",
	cex.lab=2.3,cex.axis=1.8,n=TRUE)
dev.off()

