library(mvtnorm)
library(matlab)

path.id<-"E:\\Wind Power\\Spline"
sub.id<-"Output"
dir.create(file.path(path.id,sub.id),showWarnings=F)

### Read data and codes
source(file.path(paste(path.id,"\\Code\\basisFn.r",sep="")))
source(file.path(paste(path.id,"\\Code\\mle_y.r",sep="")))

file.id<-"ILT3"
in.path1<-file.path(paste(path.id,"\\Data\\DATA_",file.id,".txt",sep=""))
in.path2<-file.path(paste(path.id,"\\Output\\sample_vs_",file.id,".txt",sep=""))
data<-read.table(in.path1)
vs.sp<-read.table(in.path2)

yrs<-c(20,50)
prob<-c(seq(0.900,0.999,by=0.001),(1-10/(yrs*365.25*24*60)))

### Set experiment parameters
n.rep<-10000
n.sp<-100
burn.in<-1000
x.init<-as.matrix(data[,1:2])
xt.init<-as.matrix(vs.sp)
y<-as.matrix(data[,3])
n<-dim(x.init)[1]
nt<-dim(xt.init)[1]
p<-dim(x.init)[2]
np<-length(prob)

### Standardize data
mean.x<-rep(1,p)
sd.x<-rep(1,p)
x<-x.init
xt<-xt.init
for(i in 1:p){
	mean.x[i]<-mean(x.init[,i])
	sd.x[i]<-sd(x.init[,i])
	x[,i]<-(x.init[,i]-mean.x[i])/sd.x[i]
	xt[,i]<-(xt.init[,i]-mean.x[i])/sd.x[i]
}

### Set MARS parameters
beta.int<-0		################################### for ILT2 only
theta.int<-0
beta.ord<-c(0,1)
theta.ord<-c(0,1)
n.int1<-length(beta.int)
n.int2<-length(theta.int)
n.ord1<-length(beta.ord)
n.ord2<-length(theta.ord)

x.mars<-matrix(1,n,1)
xt.mars<-matrix(1,nt,1)
r.mars<-matrix(1,n,1)
rt.mars<-matrix(1,nt,1)
k1<-dim(x.mars)[2]		# number of params for beta
k2<-dim(r.mars)[2]		# number of params for theta
k1.max<-50
k2.max<-20

chain_stats.ll<-matrix(0,n.rep,1)
chain_stats.k1<-matrix(0,n.rep,1)
chain_stats.k2<-matrix(0,n.rep,1)
chain_stats.xi<-matrix(0,n.rep,1)
chain_stats.q<-matrix(0,n.rep,np)

### Get initial MLE of y
res.mle<-mle.y_vs(y,x.mars,r.mars)
marg_ll<-as.numeric(res.mle[[1]])
mle.y.est<-c(res.mle[[2]])
mle.y.sd<-c((diag(res.mle[[3]])^.5))
mle.y.cov<-as.matrix(res.mle[[3]])
mle.y.res<-as.numeric(res.mle[[4]])

### MCMC replication
count<-0		### number of iterations
sample<-0		### number of samples generated

move.cnt1<-matrix(0,1,3)
move.cnt2<-matrix(0,1,3)
acc.cnt1<-matrix(0,1,3)
acc.cnt2<-matrix(0,1,3)

tic()
while(sample<n.rep){
	count<-count+1
	x.new<-as.matrix(x.mars[,1:k1])
	xt.new<-as.matrix(xt.mars[,1:k1])
	r.new<-as.matrix(r.mars[,1:k2])
	rt.new<-as.matrix(rt.mars[,1:k2])

	k1.new<-k1
	k2.new<-k2

	move1<-0		# 1:Birth 2:Death 3:Move
	move2<-0		# 1:Birth 2:Death 3:Move
	flag1<-0
	flag2<-0

	### Move selection
	u1<-runif(1,0,1)
	{if(u1<(1/3)){
		move1<-1
		flag1<-1
		if(k1==k1.max){
			move1<-3
			flag1<-3
		}
	}
	else if(u1<(2/3)){
		move1<-2
		flag1<-2
		if(k1==1){
			move1<-0
			flag1<-3
		}
	}
	else{
		move1<-3
		flag1<-3
		if(k1==1){
			move1<-0
		}
	}}

	u2<-runif(1,0,1)
	{if(u2<(1/3)){
		move2<-1
		flag2<-1
		if(k2==k2.max){
			move2<-3
			flag2<-3
		}
	}
	else if(u2<(2/3)){
		move2<-2
		flag2<-2
		if(k2==1){
			move2<-0
			flag2<-3
		}
	}
	else{
		move2<-3
		flag2<-3
		if(k2==1){
			move2<-0
		}
	}}
	
	move.cnt1[flag1]<-move.cnt1[flag1]+1
	move.cnt2[flag2]<-move.cnt2[flag2]+1

	### Take an action
	{if(move1==1){		# Birth
		k1.new<-k1+1
		id.int<-ceiling(runif(1,0,1)*n.int1)
		id.ord<-ceiling(runif(1,0,1)*n.ord1)
		basis.beta<-get.basisFn(x,xt,beta.int[id.int],beta.ord[id.ord])
		x.new<-cbind(x.new,basis.beta[[1]])
		xt.new<-cbind(xt.new,basis.beta[[2]])
	}
	else if(move1==2){	# Death
		k1.new<-k1-1
		id.del<-ceiling(runif(1,0,1)*(k1-1))+1	# select among 2 ~ k1
		x.new<-x.new[,-id.del]
		xt.new<-xt.new[,-id.del]
	}
	else if(move1==3){	# Move
		id.del<-ceiling(runif(1,0,1)*(k1-1))+1
		id.int<-ceiling(runif(1,0,1)*n.int1)
		id.ord<-ceiling(runif(1,0,1)*n.ord1)
		basis.beta<-get.basisFn(x,xt,beta.int[id.int],beta.ord[id.ord])
		x.new[,id.del]<-basis.beta[[1]]
		xt.new[,id.del]<-basis.beta[[2]]
	}}

	{if(move2==1){		# Birth
		k2.new<-k2+1
		id.int<-ceiling(runif(1,0,1)*n.int2)
		id.ord<-ceiling(runif(1,0,1)*n.ord2)
		basis.theta<-get.basisFn(x,xt,theta.int[id.int],theta.ord[id.ord])
		r.new<-cbind(r.new,basis.theta[[1]])
		rt.new<-cbind(rt.new,basis.theta[[2]])
	}
	else if(move2==2){	# Death
		k2.new<-k2-1
		id.del<-ceiling(runif(1,0,1)*(k2-1))+1	# select among 2 ~ k2
		r.new<-r.new[,-id.del]
		rt.new<-rt.new[,-id.del]
	}
	else if(move2==3){	# Move
		id.del<-ceiling(runif(1,0,1)*(k2-1))+1
		id.int<-ceiling(runif(1,0,1)*n.int2)
		id.ord<-ceiling(runif(1,0,1)*n.ord2)
		basis.theta<-get.basisFn(x,xt,theta.int[id.int],theta.ord[id.ord])
		r.new[,id.del]<-basis.theta[[1]]
		rt.new[,id.del]<-basis.theta[[2]]
	}}

	### Check acceptance of phi sub mu star
	res.mle.beta<-mle.y_vs(y,x.new,r.mars[,1:k2])
	marg_ll.beta<-as.numeric(res.mle.beta[[1]])
	mle.y.beta.est<-c(res.mle.beta[[2]])
	mle.y.beta.sd<-c((diag(res.mle.beta[[3]])^.5))
	mle.y.beta.cov<-as.matrix(res.mle.beta[[3]])
	mle.y.beta.res<-as.numeric(res.mle.beta[[4]])

	if(mle.y.beta.res<4){
		if(runif(1,0,1)<exp(marg_ll.beta-marg_ll)){
			marg_ll<-marg_ll.beta
			mle.y.est<-mle.y.beta.est
			mle.y.sd<-mle.y.beta.sd
			mle.y.cov<-mle.y.beta.cov
			k1<-k1.new
			x.mars<-as.matrix(x.new)
			xt.mars<-as.matrix(xt.new)
			acc.cnt1[flag1]<-acc.cnt1[flag1]+1
		}
	}

	### Check acceptance of phi sub sigma star
	res.mle.theta<-mle.y_vs(y,x.mars[,1:k1],r.new)
	marg_ll.theta<-as.numeric(res.mle.theta[[1]])
	mle.y.theta.est<-c(res.mle.theta[[2]])
	mle.y.theta.sd<-c((diag(res.mle.theta[[3]])^.5))
	mle.y.theta.cov<-as.matrix(res.mle.theta[[3]])
	mle.y.theta.res<-as.numeric(res.mle.theta[[4]])

	if(mle.y.theta.res<4){
		if(runif(1,0,1)<exp(marg_ll.theta-marg_ll)){
			marg_ll<-marg_ll.theta
			mle.y.est<-mle.y.theta.est
			mle.y.sd<-mle.y.theta.sd
			mle.y.cov<-mle.y.theta.cov
			k2<-k2.new
			r.mars<-as.matrix(r.new)
			rt.mars<-as.matrix(rt.new)
			acc.cnt2[flag2]<-acc.cnt2[flag2]+1
		}
	}
	
	### Take a posterior sample
	if(count>burn.in){
		beta<-rmvnorm(1,mle.y.est[1:k1],mle.y.cov[1:k1,1:k1])
		if(k2>1){
			theta<-rmvnorm(1,mle.y.est[(k1+1):(k1+k2)],
				mle.y.cov[(k1+1):(k1+k2),(k1+1):(k1+k2)])
		}
		else{
			theta<-rnorm(1,mle.y.est[(k1+1):(k1+k2)],
				mle.y.sd[(k1+1):(k1+k2)])
		}

		mu<-xt.mars[,1:k1]%*%t(beta)
		sigma<-exp(rt.mars[,1:k2]%*%t(theta))
		xi<-rnorm(1,mle.y.est[(k1+k2+1)],mle.y.sd[(k1+k2+1)])

		y.sp<-matrix(NA,nt,n.sp)
		for(i in 1:n.sp){
			y.sp[,i]<-rgev(length(mu),mu,sigma,xi)
		}
		lt<-sort(c(y.sp))[c(prob*nt*n.sp)]

		sample<-sample+1
		chain_stats.ll[sample,]<-marg_ll
		chain_stats.k1[sample,]<-k1
		chain_stats.k2[sample,]<-k2
		chain_stats.xi[sample,]<-xi
		chain_stats.q[sample,]<-lt
	}
}
toc() # 374123

write.table(x.mars,paste(path.id,"\\Output\\Xmars_",file.id,".txt",sep=""))
write.table(r.mars,paste(path.id,"\\Output\\Rmars_",file.id,".txt",sep=""))
write.table(xt.mars,paste(path.id,"\\Output\\Xtmars_",file.id,".txt",sep=""))
write.table(rt.mars,paste(path.id,"\\Output\\Rtmars_",file.id,".txt",sep=""))

### Get CI for lt
lt20<-chain_stats.q[,101]
lt50<-chain_stats.q[,102]
lt20.srt<-sort(lt20) 
lt50.srt<-sort(lt50)

CI20<-c(median(lt20),lt20.srt[250],lt20.srt[9750])
CI50<-c(median(lt50),lt50.srt[250],lt50.srt[9750])

write.table(rbind(CI20,CI50),paste(path.id,"\\Output\\CI_spline_",file.id,
	".txt",sep=""))