library(FAdist)
library(VGAM)

mle.v_W2<-function(v){
	n<-length(v)
	lik.W2<-function(param){
		shape<-param[1]
		scale<-param[2]
		-sum(log(dweibull(v,shape,scale)+10^-50))
	}
	param.init<-c(5,2)
	mle.opt<-nlm(lik.W2,param.init,hessian=T,iterlim=100)
	mle.est<-mle.opt$est
	mle.cov<-solve(mle.opt$hessian)
	mle.res<-mle.opt$code

	shape<-mle.est[1]
	scale<-mle.est[2]
	ll<-sum(log(dweibull(v,shape,scale)+10^-100))-2/2*log(n)
	list(ll,mle.est,mle.cov,mle.res)
}

pdf.W2.ref<-function(x,a,b){
	return((a*(x^(a-1))/(b^a))*exp(-((x/b)^a)))
}

mle.v_W3<-function(v){	#FAdist
	n<-length(v)
	lik.W3<-function(param){
		shape<-param[1]
		scale<-param[2]
		thres<-param[3]
		-sum(log(dweibull3(v,shape,scale,thres)+10^-50))
	}
	param.init<-c(5,2,2)
	mle.opt<-nlm(lik.W3,param.init,hessian=T,iterlim=100)
	mle.est<-mle.opt$est
	mle.cov<-solve(mle.opt$hessian)
	mle.res<-mle.opt$code

	shape<-mle.est[1]
	scale<-mle.est[2]
	thres<-mle.est[3]
	ll<-sum(log(dweibull3(v,shape,scale,thres)+10^-100))-3/2*log(n)
	list(ll,mle.est,mle.cov,mle.res)
}

pdf.W3.ref<-function(x,a,b,r){
	n<-length(x)
	pdf<-c()
	for(i in 1:n){
		if(x[i]>r){
			pdf<-c(pdf,(a*((x[i]-r)^(a-1))/(b^a))*exp(-(((x[i]-r)/b)^a)))
		}
		else{
			pdf<-c(pdf,0)
		}
	}
	return(pdf)
}

mle.v_RAY<-function(v){	#VGAM
	n<-length(v)
	lik.RAY<-function(param){
		par<-param
		-sum(log(drayleigh(v,par)+10^-50))
	}
	param.init<-c(5)
	mle.opt<-nlm(lik.RAY,param.init,hessian=T,iterlim=100)
	mle.est<-mle.opt$est
	mle.cov<-solve(mle.opt$hessian)
	mle.res<-mle.opt$code

	par<-mle.est[1]
	ll<-sum(log(drayleigh(v,par)+10^-100))-1/2*log(n)
	list(ll,mle.est,mle.cov,mle.res)
}

pdf.RAY.ref<-function(x,b){
	return((x/(b^2))*exp(-((x/b)^2)/2))
}

mle.v_LN3<-function(v){	#FAdist
	n<-length(v)
	lik.LN3<-function(param){
		shape<-param[1]
		scale<-param[2]
		thres<-param[3]
		-sum(log(dlnorm3(v,shape,scale,thres)+10^-50))
	}
	param.init<-c(2,2,2)
	mle.opt<-nlm(lik.LN3,param.init,hessian=T,iterlim=100)
	mle.est<-mle.opt$est
	mle.cov<-solve(mle.opt$hessian)
	mle.res<-mle.opt$code

	shape<-mle.est[1]
	scale<-mle.est[2]
	thres<-mle.est[3]
	ll<-sum(log(dlnorm3(v,shape,scale,thres)+10^-100))-3/2*log(n)
	list(ll,mle.est,mle.cov,mle.res)
}

pdf.LN3.ref<-function(x,m,s,r){
	n<-length(x)
	pdf<-c()
	for(i in 1:n){
		if(x[i]>r){
			pdf<-c(pdf,(1/((x[i]-r)*s*((2*pi)^(1/2))))*exp(-((log(x[i]-r)-m)^2)/(2*s^2)))
		}
		else{
			pdf<-c(pdf,0)
		}
	}
	return(pdf)
}

mle.v_G3<-function(v){	#FAdist
	n<-length(v)
	lik.G3<-function(param){
		shape<-param[1]
		scale<-param[2]
		thres<-param[3]
		-sum(log(dgamma3(v,shape,scale,thres)+10^-50))
	}
	param.init<-c(2,2,2)
	mle.opt<-nlm(lik.G3,param.init,hessian=T,iterlim=100)
	mle.est<-mle.opt$est
	mle.cov<-solve(mle.opt$hessian)
	mle.res<-mle.opt$code

	shape<-mle.est[1]
	scale<-mle.est[2]
	thres<-mle.est[3]
	ll<-sum(log(dgamma3(v,shape,scale,thres)+10^-100))-3/2*log(n)
	list(ll,mle.est,mle.cov,mle.res)
}

pdf.G3.ref<-function(x,a,b,r){
	n<-length(x)
	pdf<-c()
	for(i in 1:n){
		if(x[i]>r){
			pdf<-c(pdf,(1/(b*gamma(a)))*(((x[i]-r)/b)^(a-1))*exp(-(x[i]-r)/b))
		}
		else{
			pdf<-c(pdf,0)
		}
	}
	return(pdf)
}

mle.v_IG3<-function(v){
	n<-length(v)
	lik.IG3<-function(param){
		mu<-param[1]
		lambda<-param[2]
		gamma<-param[3]
		-sum(log(pdf.IG3.ref(v,mu,lambda,gamma)+10^-50))
	}
	param.init<-c(2,2,2)
	mle.opt<-nlm(lik.IG3,param.init,hessian=T,iterlim=100)
	mle.est<-mle.opt$est
	mle.cov<-solve(mle.opt$hessian)
	mle.res<-mle.opt$code

	mu<-mle.est[1]
	lambda<-mle.est[2]
	gamma<-mle.est[3]
	ll<-sum(log(pdf.IG3.ref(v,mu,lambda,gamma)+10^-100))-3/2*log(n)
	list(ll,mle.est,mle.cov,mle.res)
}

pdf.IG3.ref<-function(x,m,l,r){
	n<-length(x)
	pdf<-c()
	for(i in 1:n){
		if(x[i]>r){
			pdf<-c(pdf,((l/(2*pi*((x[i]-r)^3)))^(1/2))*exp(-(l*(x[i]-r-m)^2)/(2*(m^2)*(x[i]-r))))
		}
		else{
			pdf<-c(pdf,0)
		}
	}
	return(pdf)
}

