library(FAdist)
library(msm)

mle.v<-function(v){
	n<-length(v)

	lik.W3<-function(param){
		shape<-exp(param[1])
		scale<-exp(param[2])
		thres<-4					# Cut-in speed: 4 m/s
		-sum(log(dweibull3(v,shape,scale,thres)+10^-50))
	}
	param.init<-c(0.1,0.1)
	mle.opt<-nlm(lik.W3,param.init,hessian=T,iterlim=100)
	mle.est<-mle.opt$est
	mle.cov<-solve(mle.opt$hessian)
	mle.res<-mle.opt$code

	shape<-exp(mle.est[1])
	scale<-exp(mle.est[2])

	lik.mle<-sum(log(dweibull3(v,shape,scale,4)+10^-100))-2/2*log(n)

	list(lik.mle,mle.est,mle.cov,mle.res)
}

mle.s_v<-function(s,eta,delta){
	n<-length(s)
	eta<-as.matrix(eta)
	delta<-as.matrix(delta)
	n.beta<-dim(eta)[2]
	n.theta<-dim(delta)[2]

	lik.TN2<-function(param){
		beta<-param[1:n.beta]
		theta<-param[(n.beta+1):(n.beta+n.theta)]
		mu<-eta%*%beta
		sigma<-exp(delta%*%theta)
		lower<-0					# Std. dev always > 0
		-sum(log(dtnorm(s,mu,sigma,lower)+10^-100))
	}
	param.init<-rep(0.1,(n.beta+n.theta))
	mle.opt<-nlm(lik.TN2,param.init,hessian=T,iterlim=1000)
	mle.est<-mle.opt$est
	mle.cov<-matrix()
	mle.res<-mle.opt$code
	mle.itr<-mle.opt$iterations
	lik.mle<-0
	mle.hess.eigen<-c()

	cond<-1

	if(all(is.finite(mle.opt$hessian)) & !any(is.na(mle.opt$hessian)) & rcond(mle.opt$hessian)>.Machine$double.eps){
		mle.hess.eigen<-eigen(mle.opt$hessian,only.values=T)$values
		if(any(mle.hess.eigen<=.Machine$double.eps)){
			cond<-0}
	}
	else{
		cond<-0}

	if(mle.res<4 & mle.itr>10 & cond){
		mle.cov<-solve(mle.opt$hessian)
		mu<-eta%*%mle.est[1:n.beta]
		sigma<-exp(delta%*%mle.est[(n.beta+1):(n.beta+n.theta)])
		lik.mle<-sum(log(dtnorm(s,mu,sigma,0)+10^-100))-(n.beta+n.theta)/2*log(n)
	}
	else{
		mle.cov<-matrix(0,n.beta+n.theta+1,n.beta+n.theta+1)
		mle.res<-10
	}

	list(lik.mle,mle.est,mle.cov,mle.res)
}