library(mvtnorm)
library(matlab)
library(gplots)

path.id<-"E:\\Wind Power\\Spline"
sub.id<-"Output"
dir.create(file.path(path.id,sub.id),showWarnings=F)

source(file.path(paste(path.id,"\\Code\\basisFn.r",sep="")))
source(file.path(paste(path.id,"\\Code\\mle_y.r",sep="")))

### Read data
file.id<-"ILT2"

in.path<-file.path(paste(path.id,"\\Data\\DATA_",file.id,".txt",sep=""))
data<-read.table(in.path,header=T)

### Set experiment parameters
n.rep<-1000
burn.in<-100

v.ref<-seq(4,25,by=1)
s.ref<-scan()
0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9 3.0 3.1 3.2 3.3 3.4 3.5

sample.id<-sample(c(1:dim(data)[1]),10000,replace=T)
xt.init<-data[sample.id,1:2]

x.init<-as.matrix(data[,1:2])
xt.init<-as.matrix(xt.init)
y<-data[,3]
n<-dim(x.init)[1]
nt<-dim(xt.init)[1]
p<-dim(x.init)[2]

### Standardizing the data
mean.x<-rep(1,p)
sd.x<-rep(1,p)
x<-x.init
xt<-xt.init
for(k in 1:p){
	mean.x[k]<-mean(x.init[,k])
	sd.x[k]<-sd(x.init[,k])
	x[,k]<-(x.init[,k]-mean.x[k])/sd.x[k]
	xt[,k]<-(xt.init[,k]-mean.x[k])/sd.x[k]
}

### Set MARS parameters
beta.int<-0		############################# ILT2 only
theta.int<-0
beta.ord<-c(0,1)
theta.ord<-c(0,1)
n.int1<-length(beta.int)
n.int2<-length(theta.int)
n.ord1<-length(beta.ord)
n.ord2<-length(theta.ord)

x.mars<-matrix(1,n,1)
xt.mars<-matrix(1,nt,1)
r.mars<-matrix(1,n,1)
rt.mars<-matrix(1,nt,1)
k1<-dim(x.mars)[2]		# number of params for beta
k2<-dim(r.mars)[2]		# number of params for theta
k1.max<-50
k2.max<-20

chain_stats.ll<-matrix(0,n.rep,1)
chain_stats.k1<-matrix(0,n.rep,1)
chain_stats.k2<-matrix(0,n.rep,1)
chain_stats.xi<-matrix(0,n.rep,1)
chain.v<-matrix(0,3,length(v.ref))	# contain CI data of v
chain.s<-matrix(0,3,length(s.ref))	# contain CI data of s

### Get initial MLE of y
res.mle<-mle.y_vs(y,x.mars,r.mars)
marg_ll<-as.numeric(res.mle[[1]])
mle.y.est<-c(res.mle[[2]])
mle.y.sd<-c((diag(res.mle[[3]])^.5))
mle.y.cov<-as.matrix(res.mle[[3]])
mle.y.res<-as.numeric(res.mle[[4]])

### MCMC replication
count<-0		# for number of replications including burn-in
sample<-0		# for number of replications after burn-in

move.cnt1<-matrix(0,1,3)
move.cnt2<-matrix(0,1,3)
acc.cnt1<-matrix(0,1,3)
acc.cnt2<-matrix(0,1,3)

tic()
while(sample<n.rep){
	count<-count+1
	x.new<-as.matrix(x.mars[,1:k1])
	xt.new<-as.matrix(xt.mars[,1:k1])
	r.new<-as.matrix(r.mars[,1:k2])
	rt.new<-as.matrix(rt.mars[,1:k2])

	k1.new<-k1
	k2.new<-k2

	move1<-0		# 1:Birth 2:Death 3:Move
	move2<-0		# 1:Birth 2:Death 3:Move
	flag1<-0
	flag2<-0

	### Move selection
	u1<-runif(1,0,1)
	{if(u1<(1/3)){
		move1<-1
		flag1<-1
		if(k1==k1.max){
			move1<-3
			flag1<-3
		}
	}
	else if(u1<(2/3)){
		move1<-2
		flag1<-2
		if(k1==1){
			move1<-0
			flag1<-3
		}
	}
	else{
		move1<-3
		flag1<-3
		if(k1==1){
			move1<-0
		}
	}}

	u2<-runif(1,0,1)
	{if(u2<(1/3)){
		move2<-1
		flag2<-1
		if(k2==k2.max){
			move2<-3
			flag2<-3
		}
	}
	else if(u2<(2/3)){
		move2<-2
		flag2<-2
		if(k2==1){
			move2<-0
			flag2<-3
		}
	}
	else{
		move2<-3
		flag2<-3
		if(k2==1){
			move2<-0
		}
	}}
	
	move.cnt1[flag1]<-move.cnt1[flag1]+1
	move.cnt2[flag2]<-move.cnt2[flag2]+1

	### Take an action
	{if(move1==1){		# Birth
		k1.new<-k1+1
		id.int<-ceiling(runif(1,0,1)*n.int1)
		id.ord<-ceiling(runif(1,0,1)*n.ord1)
		basis.beta<-get.basisFn(x,xt,beta.int[id.int],beta.ord[id.ord])
		x.new<-cbind(x.new,basis.beta[[1]])
		xt.new<-cbind(xt.new,basis.beta[[2]])
	}
	else if(move1==2){	# Death
		k1.new<-k1-1
		id.del<-ceiling(runif(1,0,1)*(k1-1))+1	# select among 2~k1
		x.new<-x.new[,-id.del]
		xt.new<-xt.new[,-id.del]
	}
	else if(move1==3){	# Move
		id.del<-ceiling(runif(1,0,1)*(k1-1))+1
		id.int<-ceiling(runif(1,0,1)*n.int1)
		id.ord<-ceiling(runif(1,0,1)*n.ord1)
		basis.beta<-get.basisFn(x,xt,beta.int[id.int],beta.ord[id.ord])
		x.new[,id.del]<-basis.beta[[1]]
		xt.new[,id.del]<-basis.beta[[2]]
	}}

	{if(move2==1){		# Birth
		k2.new<-k2+1
		id.int<-ceiling(runif(1,0,1)*n.int2)
		id.ord<-ceiling(runif(1,0,1)*n.ord2)
		basis.theta<-get.basisFn(as.matrix(x[,1]),as.matrix(xt[,1]),
			theta.int[id.int],theta.ord[id.ord])	# not consider s, ILT2 only
		r.new<-cbind(r.new,basis.theta[[1]])
		rt.new<-cbind(rt.new,basis.theta[[2]])
	}
	else if(move2==2){	# Death
		k2.new<-k2-1
		id.del<-ceiling(runif(1,0,1)*(k2-1))+1	# select among 2~k2
		r.new<-r.new[,-id.del]
		rt.new<-rt.new[,-id.del]
	}
	else if(move2==3){	# Move
		id.del<-ceiling(runif(1,0,1)*(k2-1))+1
		id.int<-ceiling(runif(1,0,1)*n.int2)
		id.ord<-ceiling(runif(1,0,1)*n.ord2)
		basis.theta<-get.basisFn(as.matrix(x[,1]),as.matrix(xt[,1]),
			theta.int[id.int],theta.ord[id.ord])	# not consider s, ILT2 only
		r.new[,id.del]<-basis.theta[[1]]
		rt.new[,id.del]<-basis.theta[[2]]
	}}

	### Check acceptance of phi sub mu star
	res.mle.beta<-mle.y_vs(y,x.new,r.mars[,1:k2])
	marg_ll.beta<-as.numeric(res.mle.beta[[1]])
	mle.y.beta.est<-c(res.mle.beta[[2]])
	mle.y.beta.sd<-c((diag(res.mle.beta[[3]])^.5))
	mle.y.beta.cov<-as.matrix(res.mle.beta[[3]])
	mle.y.beta.res<-as.numeric(res.mle.beta[[4]])

	if(mle.y.beta.res<4){
		if(runif(1,0,1)<exp(marg_ll.beta-marg_ll)){
			marg_ll<-marg_ll.beta
			mle.y.est<-mle.y.beta.est
			mle.y.sd<-mle.y.beta.sd
			mle.y.cov<-mle.y.beta.cov
			k1<-k1.new
			x.mars<-as.matrix(x.new)
			xt.mars<-as.matrix(xt.new)
			acc.cnt1[flag1]<-acc.cnt1[flag1]+1
		}
	}

	### Check acceptance of phi sub sigma star
	res.mle.theta<-mle.y_vs(y,x.mars[,1:k1],r.new)
	marg_ll.theta<-as.numeric(res.mle.theta[[1]])
	mle.y.theta.est<-c(res.mle.theta[[2]])
	mle.y.theta.sd<-c((diag(res.mle.theta[[3]])^.5))
	mle.y.theta.cov<-as.matrix(res.mle.theta[[3]])
	mle.y.theta.res<-as.numeric(res.mle.theta[[4]])

	if(mle.y.theta.res<4){
		if(runif(1,0,1)<exp(marg_ll.theta-marg_ll)){
			marg_ll<-marg_ll.theta
			mle.y.est<-mle.y.theta.est
			mle.y.sd<-mle.y.theta.sd
			mle.y.cov<-mle.y.theta.cov
			k2<-k2.new
			r.mars<-as.matrix(r.new)
			rt.mars<-as.matrix(rt.new)
			acc.cnt2[flag2]<-acc.cnt2[flag2]+1
		}
	}

	### Take posterior samples
	if(count>burn.in){
		beta<-rmvnorm(1,mle.y.est[1:k1],mle.y.cov[1:k1,1:k1])
		if(k2>1){
			theta<-rmvnorm(1,mle.y.est[(k1+1):(k1+k2)],
				mle.y.cov[(k1+1):(k1+k2),(k1+1):(k1+k2)])
		}
		else{
			theta<-rnorm(1,mle.y.est[(k1+1):(k1+k2)],
				mle.y.sd[(k1+1):(k1+k2)])
		}

		mu<-xt.mars[,1:k1]%*%t(beta)
		sigma<-exp(rt.mars[,1:k2]%*%t(theta))
		xi<-rnorm(1,mle.y.est[(k1+k2+1)],mle.y.sd[(k1+k2+1)])

		for(j in 1:length(v.ref)){
			loc<-which(round(xt.init[,1])==v.ref[j])
			if(length(loc)>0){
				y.sp<-rgev(length(loc)*100,mu[loc],sigma[loc],xi)
				chain.v[,j]<-c(quantile(y.sp,0.025),mean(y.sp),
					quantile(y.sp,0.975))
			}
		}

		for(j in 1:length(s.ref)){
			loc<-which(round(xt.init[,2],1)==s.ref[j])
			if(length(loc)>0){
				y.sp<-rgev(length(loc)*100,mu[loc],sigma[loc],xi)
				chain.s[,j]<-c(quantile(y.sp,0.025),mean(y.sp),
					quantile(y.sp,0.975))
			}
		}

		{if(sample==0){
			chain.v.store<-chain.v
			chain.s.store<-chain.s
		}
			
		{chain.v.store<-(chain.v.store*sample+chain.v)/(sample+1)
		chain.s.store<-(chain.s.store*sample+chain.s)/(sample+1)}}
			
		sample<-sample+1
		chain_stats.ll[sample,]<-marg_ll
		chain_stats.k1[sample,]<-k1
		chain_stats.k2[sample,]<-k2
		chain_stats.xi[sample,]<-xi
	}
}
toc()

uCI.v<-chain.v.store[3,]-chain.v.store[2,]
lCI.v<-chain.v.store[2,]-chain.v.store[1,]

uCI.s<-chain.s.store[3,]-chain.s.store[2,]
lCI.s<-chain.s.store[2,]-chain.s.store[1,]

### Figure 5
v.loc<-seq(1,19)
path1<-file.path(paste(path.id,"\\Output\\Fig5_",file.id,".pdf",
	sep=""))
pdf(path1)
par(mar=c(6,6,5,3))
plot(x.init[,1],y,xlab="10-min average wind speed (m/s)",
	ylab="10-min maximum load (MN-m)",xlim=c(4,22),ylim=c(0,0.6),
	cex.lab=2.3,cex.axis=1.8,cex=1.2,lwd=2)
plotCI(v.ref[v.loc],chain.v.store[2,v.loc],uiw=uCI.v[v.loc],
	liw=lCI.v[v.loc],add=T,col="red",pch=2,lwd=6)
legend("topleft",c("observed data","posterior predictive mean"),
	pch=c(1,2),cex=2.0,pt.lwd=3,col=c(1,2))
dev.off()

### Figure 6
s.loc<-seq(3,33,by=2)
path2<-file.path(paste(path.id,"\\Output\\Fig6_",file.id,".pdf",
	sep=""))
pdf(path2)
par(mar=c(6,6,5,3))
plot(x.init[,2],y,xlab="standard deviation during 10-min (m/s)",
	ylab="10-min maximum load (MN-m)",xlim=c(0.3,3.5),ylim=c(0,0.6),
	cex.lab=2.2,cex.axis=1.8,cex=1.2,lwd=2)
plotCI(s.ref[s.loc],chain.s.store[2,s.loc],uiw=uCI.s[s.loc],
	liw=lCI.s[s.loc],add=T,col="red",pch=2,lwd=6)
legend("topleft",c("observed data","posterior predictive mean"),
	pch=c(1,2),cex=2.0,pt.lwd=3,col=c(1,2))
dev.off()







