library(VGAM)
library(mnormt)
library(matlab)

path.id<-"E:\\Wind Power\\Spline"
sub.id<-"Output"
dir.create(file.path(path.id,sub.id),showWarnings=F)

data<-read.table(file.path(paste(path.id,"\\Data\\sim_dataTR.txt",sep="")))
ws<-read.table(file.path(paste(path.id,"\\Output\\sample_v_sim.txt",sep="")))

bin.size<-2
n.rep<-10000	# To construct credible interval (M sub l)
n.sp<-100		# To sample y from gumbel given mu and sigma (N sub l)
prob.t<-c(0.0001,0.00001)

### Division of bins
bin.div<-function(data,bin.size){
	v<-data[,1]
	n<-length(v)
	n.bin<-ceiling((24-5)/bin.size)
	ref<-seq(5,24,by=bin.size)
	index<-trunc((v-4)/bin.size)+1
	index[which(index>n.bin)]<-n.bin
	data.new<-cbind(index,data)
	list(data.new,ref)
}

### Sample parameters from the MLE distributions
sample.MLE<-function(data,bin.size,n.rep){
	bin<-bin.div(data,bin.size)
	data.new<-bin[[1]]
	ref<-bin[[2]]
	param.est<-matrix(0,length(ref),2*n.rep)
	bin.info<-matrix(0,length(ref),2)
	for(i in 1:length(ref)){
		t<-which(data.new[,1]==i)
		data.bin<-data.new[t,]
		if(dim(data.bin)[1]>5){
			lik.Gum<-function(param){
				mu<-param[1]
				sigma<-exp(param[2])
				-sum(log(dgumbel(data.bin[,3],mu,sigma)))
			}
			param.init<-rep(0.1,2)
			mle.Gum<-nlm(lik.Gum,param.init,hessian=T,stepmax=10,
				iterlim=1000)
			bin.info[i,2]<-dim(data.bin)[1]
			samples<-rmnorm(n.rep,mle.Gum$est,
				solve(mle.Gum$hessian))
			param.est[i,]<-c(samples[,1],exp(samples[,2]))
		}
	}
	bin.info[,1]<-ref
	params<-neighbor.est(list(bin.info,param.est))
	return(params)
}

### Estimate parameters for the empty (or <=5) bins
neighbor.est<-function(samples){
	ref<-samples[[1]]
	params<-samples[[2]]
	index.exist<-which(ref[,2]>5)
	index.empty<-which(ref[,2]<=5)
	bin.exist<-ref[index.exist,]
	n.exist<-dim(bin.exist)[1]

	for(i in index.empty){
		loc<-matrix(rep((ref[i,1]),n.exist),n.exist,1,byrow=T)
		dist<-1/((((bin.exist[,1]-loc[,1])/bin.size[1])^2)^0.5)
		weight<-dist/sum(dist)
		params[i,]<-sum(weight*params[index.exist,])
	}
	return(params)
}

param.sp<-sample.MLE(data,bin.size,n.rep)	# Sample params 4 each bin from MLE
bin.ts<-bin.div(ws,bin.size)[[1]]	# Get the bin index for each sample pt
n.obs<-dim(bin.ts)[1]
lt<-matrix(0,n.rep,2)

tic()
for(i in 1:n.rep){
	mu<-param.sp[bin.ts[,1],i]
	sigma<-param.sp[bin.ts[,1],n.rep+i]
	y.est<-rgumbel(n.sp*n.obs,mu,sigma)
	y.srt<-sort(y.est,decreasing=T)
	lt[i,]<-y.srt[prob.t*n.sp*n.obs]
}
toc()

### Results
lt1<-lt[,1]		# lt with prob 0.0001
lt2<-lt[,2]		# lt with prob 0.00001

write.table(cbind(lt1,lt2),paste(path.id,"\\Output\\Dist_sim_bin.txt",sep=""))
