library(FAdist)
library(msm)
library(matlab)

path.id<-"E:\\Wind Power\\Spline"
sub.id<-"Output"
dir.create(file.path(path.id,sub.id),showWarnings=F)

### Set parameters
shape<-1.318	# How to get these parameter values?
scale<-5.429

n.TR<-1000
n.TS_rep<-100
n.TS_data<-100000
n.sp<-1000

### Simulate training datasets
ws_TR<-rweibull3(1.1*n.TR,shape,scale,4)
ws.ref_TR<-ws_TR[sample(which(ws_TR<25),n.TR)]

mu_TR<-rep(0,n.sp)
sigma_TR<-rep(0,n.sp)
lt_TR<-matrix(NA,n.TR,1)

tic()
for(i in 1:n.TR){
	v<-rnorm(n.sp,ws.ref_TR[i],ws.ref_TR[i]*0.1)
	{if(mean(v)>17){
		const<-500-1.6*(mean(v)+mean(v)^2)
	}
	else{
		const<-0
	}}

	for(j in 1:n.sp){
		mu_TR[j]<-1500/(1+48*exp(-0.3*v[j]))+const
		sigma_TR[j]<-100*log(v[j])
	}
	load_sp<-rtnorm(n.sp,mu_TR,sigma_TR,lower=0)+rnorm(n.sp,0,1)
	lt_TR[i]<-max(load_sp)/1000
}
toc()

data.TR<-as.data.frame(cbind(ws.ref_TR,lt_TR))
names(data.TR)<-c("x","y")
write.table(data.TR,paste(path.id,"\\Data\\sim_dataTR.txt",sep=""))

### Figure 9(a)
pdf(paste(path.id,"\\Output\\Fig9(a).pdf",sep=""))
par(mar=c(5,5,3,5))
plot(data.TR[,1],data.TR[,2],xlab="x",ylab="y")
dev.off()

### Simulate testing datasets
ws_TS<-rweibull3(1.1*(n.TS_rep*n.TS_data),shape,scale,4)
ws.ref_TS<-ws_TS[sample(which(ws_TS<25),(n.TS_rep*n.TS_data))]

lt_TS<-rep(NA,(n.TS_data*n.TS_rep))

tic()
for(ind in 1:(n.TS_data*n.TS_rep)){
	mu_TS<-rep(0,n.sp)
	sigma_TS<-rep(0,n.sp)

	v<-rnorm(n.sp,ws.ref_TS[ind],ws.ref_TS[ind]*0.1)
	{if(mean(v)>17){
		const<-500-1.6*(mean(v)+mean(v)^2)
	}
	else{
		const<-0
	}}

	mu_TS<-1500/(1+48*exp(-0.3*v))+const
	sigma_TS<-100*log(v)

	load_sp<-rtnorm(n.sp,mu_TS,sigma_TS,lower=0)+rnorm(n.sp,0,1)
	lt_TS[ind]<-max(load_sp)/1000
}
toc()

lt.obs<-as.data.frame(cbind(ws.ref_TS,lt_TS))
write.table(lt.obs,paste(path.id,"\\Data\\sim_dataTS.txt",sep=""))

### Get CI for the simulated observation of extreme loads
lt.data<-matrix(c(lt_TS),n.TS_data,n.TS_rep)
lt.srt<-apply(lt.data,2,sort)
lt1.obs<-lt.srt[99990,]
lt2.obs<-lt.srt[99999,]
write.table(cbind(lt1.obs,lt2.obs),paste(path.id,"\\Output\\Dist_sim_obs.txt",
	sep=""))



