library(mvtnorm)
library(matlab)

path.id<-"E:\\Wind Power\\Spline"
sub.id<-"Output"
dir.create(file.path(path.id,sub.id),showWarnings=F)

### Read data and codes
source(file.path(paste(path.id,"\\Code\\mle_vs.r",sep="")))

data<-read.table(file.path(paste(path.id,"\\Data\\sim_dataTR.txt",sep="")))

v.data<-as.matrix(data[,1])

### Set experiment parameters
n.rep<-1000
n.sp<-100
v.cutIn<-4
v.cutOut<-25

### Get MLE of v and get posterior samples based on MLEs
mle.v<-mle.v(v.data)
mle.v.est<-mle.v[[2]]
mle.v.var<-mle.v[[3]]

post.sample<-rmvnorm(10*n.rep,mle.v.est,mle.v.var)
id_select<-sample(c(1:(10*n.rep)),n.rep)
v_par.shape<-exp(post.sample[id_select,1])
v_par.scale<-exp(post.sample[id_select,2])
v_par.thres<-v.cutIn

### Get wind speed samples
v.sample<-matrix(NA,n.rep,1.2*n.sp)	# to delete v's over cut in and out

for(i in 1:n.rep){
	v.sample[i,]<-rweibull3(1.2*n.sp,v_par.shape,v_par.scale,v_par.thres)
}

v.sample<-c(v.sample)
v.sample<-v.sample[sample(which(v.sample<v.cutOut),n.rep*n.sp)]

write.table(v.sample,paste(path.id,"\\Output\\sample_v_sim.txt",sep=""))