library(mvtnorm)
library(matlab)

path.id<-"E:\\Wind Power\\Spline"
sub.id<-"Output"
dir.create(file.path(path.id,sub.id),showWarnings=F)

### Read data and codes
source(file.path(paste(path.id,"\\Code\\basisFn.r",sep="")))
source(file.path(paste(path.id,"\\Code\\mle_vs.r",sep="")))

file.id<-"ILT3"
in.path<-file.path(paste(path.id,"\\Data\\DATA_",file.id,".txt",sep=""))
data<-read.table(in.path)

v.data<-as.matrix(data[,1])
s.data<-as.matrix(data[,2])

### Set experiment parameters
n.rep<-1000
n.sp<-100
v.cutIn<-4
v.cutOut<-25

### Get MLE of v and get posterior samples based on MLEs
mle.v<-mle.v(v.data)

get_ll.W3<-function(param){
	shape<-exp(param[1])
	scale<-exp(param[2])
	thres<-v.cutIn
	sum(log(dweibull3(v.data,shape,scale,thres)+10^-100))
}

mcmc.v<-function(v.data,mle.v,n.rep.v){
	n.obs<-nrow(v.data)
	v.init<-c(mle.v[[2]])
	var.v<-mle.v[[3]]
	param_mcmc.v<-matrix(NA,n.rep.v+1,2)
	param_mcmc.v[1,]<-c(v.init)

	for(i in 1:n.rep.v){
		param.v_new<-rmvnorm(1,param_mcmc.v[i,],var.v)
		marg_ll<-get_ll.W3(param_mcmc.v[i,])-(2)/2*log(n.obs)
		marg_ll.new<-get_ll.W3(param.v_new)-(2)/2*log(n.obs)

		if(runif(1,0,1)<exp(marg_ll.new-marg_ll)){
			param_mcmc.v[(i+1),]<-param.v_new
		}
		else{
			param_mcmc.v[(i+1),]<-param_mcmc.v[i,]
		}
	}
	return(param_mcmc.v[-1,])
}

post.sample<-mcmc.v(v.data,mle.v,10*n.rep)
id_select<-sample(c(1:(10*n.rep)),n.rep)
v_par.shape<-exp(post.sample[id_select,1])
v_par.scale<-exp(post.sample[id_select,2])
v_par.thres<-v.cutIn

### Get wind speed samples
v.sample<-matrix(NA,n.rep,1.2*n.sp)	# to delete v's over cut out

for(i in 1:n.rep){
	v.sample[i,]<-rweibull3(1.2*n.sp,v_par.shape,v_par.scale,v_par.thres)
}

v.sample<-c(v.sample)
v.sample<-v.sample[sample(which(v.sample<v.cutOut),n.rep*n.sp)]

### Set parameters
x.init<-as.matrix(v.data)
xt.init<-as.matrix(v.sample)
y<-as.matrix(s.data)

#############################################################

burn.in<-1000
n<-dim(x.init)[1]
nt<-dim(xt.init)[1]
p<-dim(x.init)[2]

### Standardize data
mean.x<-rep(1,p)
sd.x<-rep(1,p)
x<-x.init
xt<-xt.init
for(i in 1:p){
	mean.x[i]<-mean(x.init[,i])
	sd.x[i]<-sd(x.init[,i])
	x[,i]<-(x.init[,i]-mean.x[i])/sd.x[i]
	xt[,i]<-(xt.init[,i]-mean.x[i])/sd.x[i]
}

### Set MARS parameters
beta.int<-0
theta.int<-0
beta.ord<-c(0,1)
theta.ord<-c(0,1)
n.int1<-length(beta.int)
n.int2<-length(theta.int)
n.ord1<-length(beta.ord)
n.ord2<-length(theta.ord)

k1.max<-50
k2.max<-20
x.mars<-matrix(1,n,1)
xt.mars<-matrix(1,nt,1)
r.mars<-matrix(1,n,1)
rt.mars<-matrix(1,nt,1)
k1<-1		# number of params for beta
k2<-1		# number of params for theta

chain_stats.ll<-matrix(0,n.rep,1)
chain_stats.k1<-matrix(0,n.rep,1)
chain_stats.k2<-matrix(0,n.rep,1)
chain_stats.eta<-matrix(0,n.rep,n.rep*n.sp)
chain_stats.delta<-matrix(0,n.rep,n.rep*n.sp)

### Get initial MLE of s
res.mle<-mle.s_v(y,x.mars[,1:k1],r.mars[,1:k2])
marg_ll<-as.numeric(res.mle[[1]])
mle.s.est<-c(res.mle[[2]])
mle.s.sd<-c((diag(res.mle[[3]])^.5))
mle.s.cov<-as.matrix(res.mle[[3]])
mle.s.res<-as.numeric(res.mle[[4]])

### MCMC replication
count<-0		### number of iterations
sample<-0		### number of samples generated

move.cnt1<-matrix(0,1,3)
move.cnt2<-matrix(0,1,3)
acc.cnt1<-matrix(0,1,3)
acc.cnt2<-matrix(0,1,3)

tic()
while(sample<n.rep){
	count<-count+1
	x.new<-as.matrix(x.mars[,1:k1])
	xt.new<-as.matrix(xt.mars[,1:k1])
	r.new<-as.matrix(r.mars[,1:k2])
	rt.new<-as.matrix(rt.mars[,1:k2])

	k1.new<-k1
	k2.new<-k2

	move1<-0		# 1:Birth 2:Death 3:Move
	move2<-0		# 1:Birth 2:Death 3:Move
	flag1<-0
	flag2<-0

	### Move selection
	u1<-runif(1,0,1)
	if(u1<(1/3)){
		move1<-1
		flag1<-1
		if(k1==k1.max){
			move1<-3
			flag1<-3
		}
	}
	else if(u1<(2/3)){
		move1<-2
		flag1<-2
		if(k1==1){
			move1<-0
			flag1<-3
		}
	}
	else{
		move1<-3
		flag1<-3
		if(k1==1){
			move1<-0
		}
	}

	u2<-runif(1,0,1)
	if(u2<(1/3)){
		move2<-1
		flag2<-1
		if(k2==k2.max){
			move2<-3
			flag2<-3
		}
	}
	else if(u2<(2/3)){
		move2<-2
		flag2<-2
		if(k2==1){
			move2<-0
			flag2<-3
		}
	}
	else{
		move2<-3
		flag2<-3
		if(k2==1){
			move2<-0
		}
	}
	
	move.cnt1[flag1]<-move.cnt1[flag1]+1
	move.cnt2[flag2]<-move.cnt2[flag2]+1

	### Take an action
	if(move1==1){		# Birth
		k1.new<-k1+1
		id.int<-ceiling(runif(1,0,1)*n.int1)
		id.ord<-ceiling(runif(1,0,1)*n.ord1)
		basis.beta<-get.basisFn(x,xt,beta.int[id.int],beta.ord[id.ord])
		x.new<-cbind(x.new,basis.beta[[1]])
		xt.new<-cbind(xt.new,basis.beta[[2]])
	}
	else if(move1==2){	# Death
		k1.new<-k1-1
		id.del<-ceiling(runif(1,0,1)*(k1-1))+1	# select among 2 ~ k1
		x.new<-x.new[,-id.del]
		xt.new<-xt.new[,-id.del]
	}
	else if(move1==3){	# Move
		id.del<-ceiling(runif(1,0,1)*(k1-1))+1
		id.int<-ceiling(runif(1,0,1)*n.int1)
		id.ord<-ceiling(runif(1,0,1)*n.ord1)
		basis.beta<-get.basisFn(x,xt,beta.int[id.int],beta.ord[id.ord])
		x.new[,id.del]<-basis.beta[[1]]
		xt.new[,id.del]<-basis.beta[[2]]
	}

	if(move2==1){		# Birth
		k2.new<-k2+1
		id.int<-ceiling(runif(1,0,1)*n.int2)
		id.ord<-ceiling(runif(1,0,1)*n.ord2)
		basis.theta<-get.basisFn(x,xt,theta.int[id.int],theta.ord[id.ord])
		r.new<-cbind(r.new,basis.theta[[1]])
		rt.new<-cbind(rt.new,basis.theta[[2]])
	}
	else if(move2==2){	# Death
		k2.new<-k2-1
		id.del<-ceiling(runif(1,0,1)*(k2-1))+1	# select among 2 ~ k2
		r.new<-r.new[,-id.del]
		rt.new<-rt.new[,-id.del]
	}
	else if(move2==3){	# Move
		id.del<-ceiling(runif(1,0,1)*(k2-1))+1
		id.int<-ceiling(runif(1,0,1)*n.int2)
		id.ord<-ceiling(runif(1,0,1)*n.ord2)
		basis.theta<-get.basisFn(x,xt,theta.int[id.int],theta.ord[id.ord])
		r.new[,id.del]<-basis.theta[[1]]
		rt.new[,id.del]<-basis.theta[[2]]
	}

	### Check acceptance of phi sub mu star
	res.mle.beta<-mle.s_v(y,x.new,r.mars[,1:k2])
	marg_ll.beta<-as.numeric(res.mle.beta[[1]])
	mle.s.beta.est<-c(res.mle.beta[[2]])
	mle.s.beta.sd<-c((diag(res.mle.beta[[3]])^.5))
	mle.s.beta.cov<-as.matrix(res.mle.beta[[3]])
	mle.s.beta.res<-as.numeric(res.mle.beta[[4]])

	if(mle.s.beta.res<4){
		if(runif(1,0,1)<exp(marg_ll.beta-marg_ll)){
			marg_ll<-marg_ll.beta
			mle.s.est<-mle.s.beta.est
			mle.s.sd<-mle.s.beta.sd
			mle.s.cov<-mle.s.beta.cov
			k1<-k1.new
			x.mars<-as.matrix(x.new)
			xt.mars<-as.matrix(xt.new)
			acc.cnt1[flag1]<-acc.cnt1[flag1]+1
		}
	}

	### Check acceptance of phi sub sigma star
	res.mle.theta<-mle.s_v(y,x.mars[,1:k1],r.new)
	marg_ll.theta<-as.numeric(res.mle.theta[[1]])
	mle.s.theta.est<-c(res.mle.theta[[2]])
	mle.s.theta.sd<-c((diag(res.mle.theta[[3]])^.5))
	mle.s.theta.cov<-as.matrix(res.mle.theta[[3]])
	mle.s.theta.res<-as.numeric(res.mle.theta[[4]])

	if(mle.s.theta.res<4){
		if(runif(1,0,1)<exp(marg_ll.theta-marg_ll)){
			marg_ll<-marg_ll.theta
			mle.s.est<-mle.s.theta.est
			mle.s.sd<-mle.s.theta.sd
			mle.s.cov<-mle.s.theta.cov
			k2<-k2.new
			r.mars<-as.matrix(r.new)
			rt.mars<-as.matrix(rt.new)
			acc.cnt2[flag2]<-acc.cnt2[flag2]+1
		}
	}

	### Take a posterior sample
	if(count>burn.in){
		beta<-rmvnorm(1,mle.s.est[1:k1],mle.s.cov[1:k1,1:k1])
		if(k2>1){
			theta<-rmvnorm(1,mle.s.est[(k1+1):(k1+k2)],
				mle.s.cov[(k1+1):(k1+k2),(k1+1):(k1+k2)])
		}
		else{
			theta<-rnorm(1,mle.s.est[(k1+1):(k1+k2)],
				mle.s.sd[(k1+1):(k1+k2)])
		}

		eta<-xt.mars[,1:k1]%*%t(beta)
		delta<-exp(rt.mars[,1:k2]%*%t(theta))

		sample<-sample+1
		chain_stats.ll[sample,]<-marg_ll
		chain_stats.k1[sample,]<-k1
		chain_stats.k2[sample,]<-k2
		chain_stats.eta[sample,]<-eta
		chain_stats.delta[sample,]<-delta
	}
}
toc()

{if(length(which(is.na(chain_stats.eta[,1])==1))>0){
	id.na<-which(is.na(chain_stats.eta[,1])==1)
	eta.post<-apply(chain_stats.eta[-id.na,],2,mean)
	delta.post<-apply(chain_stats.delta[-id.na,],2,mean)
}
else{
	eta.post<-apply(chain_stats.eta,2,mean)
	delta.post<-apply(chain_stats.delta,2,mean)
}}

#############################################################

s.sample<-rtnorm(n.rep*n.sp,eta.post,delta.post,0)

vs.sample<-cbind(v.sample,s.sample)

write.table(vs.sample,paste(path.id,"\\Output\\sample_vs_",file.id,".txt",
	sep=""))
